@@ -18,7 +18,7 @@
 
 #include "analitzautils.h"
 
-#include "abstractexpressionvisitor.h"
+#include "expressionwriter.h"
 #include "vector.h"
 #include "value.h"
 #include "list.h"
@@ -29,7 +29,6 @@
 #include "apply.h"
 #include <QVariant>
 #include "customobject.h"
-#include "matrix.h"
 
 using namespace Analitza;
 namespace AnalitzaUtils
@@ -46,12 +45,6 @@ QStringList dependencies(const Object* o, const QStringList& scope)
 			if(!scope.contains(i->name()))
 				ret += i->name();
 		}	break;
-		case Object::matrix: {
-			const Matrix *v=(const Matrix*) o;
-			for(Matrix::const_iterator it=v->constBegin(); it!=v->constEnd(); ++it) {
-				ret += dependencies(*it, scope).toSet();
-			}
-		}	break;
 		case Object::vector: {
 			const Vector *v=(const Vector*) o;
 			for(Vector::const_iterator it=v->constBegin(); it!=v->constEnd(); ++it) {
@@ -64,12 +57,6 @@ QStringList dependencies(const Object* o, const QStringList& scope)
 				ret += dependencies(*it, scope).toSet();
 			}
 		}	break;
-		case Object::matrixrow: {
-			const MatrixRow *v=(const MatrixRow*) o;
-			for(MatrixRow::const_iterator it=v->constBegin(); it!=v->constEnd(); ++it) {
-				ret += dependencies(*it, scope).toSet();
-			}
-		}	break;
 		case Object::container: {
 			const Container *c = static_cast<const Container*>(o);
 			int skip=c->bvarCount();
@@ -128,20 +115,6 @@ bool hasTheVar(const QSet<QString> & vars, const Object * o)
 				found |= hasTheVar(vars, *it);
 			}
 		}	break;
-		case Object::matrix: {
-			const Matrix *v=static_cast<const Matrix*>(o);
-			Matrix::const_iterator it, itEnd=v->constEnd();
-			for(it=v->constBegin(); it!=itEnd; ++it) {
-				found |= hasTheVar(vars, *it);
-			}
-		}	break;
-		case Object::matrixrow: {
-			const MatrixRow *v=static_cast<const MatrixRow*>(o);
-			MatrixRow::const_iterator it, itEnd=v->constEnd();
-			for(it=v->constBegin(); it!=itEnd; ++it) {
-				found |= hasTheVar(vars, *it);
-			}
-		}	break;
 		case Object::list: {
 			const List *v=static_cast<const List*>(o);
 			List::const_iterator it, itEnd=v->constEnd();
@@ -223,18 +196,6 @@ bool hasVars(const Analitza::Object* o, const QStringList& bvars)
 				r |= hasVars(*it, bvars);
 			}
 		}	break;
-		case Object::matrix: {
-			Matrix *v=(Matrix*) o;
-			for(Matrix::const_iterator it=v->constBegin(); it!=v->constEnd(); ++it) {
-				r |= hasVars(*it, bvars);
-			}
-		}	break;
-		case Object::matrixrow: {
-			MatrixRow *v=(MatrixRow*) o;
-			for(MatrixRow::const_iterator it=v->constBegin(); it!=v->constEnd(); ++it) {
-				r |= hasVars(*it, bvars);
-			}
-		}	break;
 		case Object::list: {
 			List *v=(List*) o;
 			for(List::const_iterator it=v->constBegin(); it!=v->constEnd(); ++it) {
@@ -280,14 +241,14 @@ bool hasVars(const Analitza::Object* o, const QStringList& bvars)
 	return r;
 }
 
-struct ObjectWalker : public AbstractExpressionVisitor
+struct ObjectWalker : public ExpressionWriter
 {
 	ObjectWalker(const QByteArray& pref) : ind(0), m_prefix(pref) {}
 	
-	virtual QVariant visit(const Operator* root)
+	virtual QString accept(const Operator* root)
 	{ qDebug() << prefix().constData() << "| operator: " << root->toString(); return QString(); }
 	
-	virtual QVariant visit(const Ci* var)
+	virtual QString accept(const Ci* var)
  	{
 		QString value;
 		if(var->depth()>=0)
@@ -299,14 +260,14 @@ struct ObjectWalker : public AbstractExpressionVisitor
 		return QString();
 	}
 	
-	virtual QVariant visit(const Cn* num)
+	virtual QString accept(const Cn* num)
 	{ qDebug() << prefix().constData() << "| num: " << num->value() << " format: " << num->format(); return QString(); }
 	
-	virtual QVariant visit(const CustomObject* c)
+	virtual QString accept(const CustomObject* c)
 	{ qDebug() << prefix().constData() << "| custom " << c; return QString(); }
 	
 	
-	virtual QVariant visit(const Container* c)
+	virtual QString accept(const Container* c)
 	{
 		qDebug() << prefix().constData() << "| cont: " << c->tagName();// << "=" << c->toString();
 		ind++;
@@ -316,7 +277,7 @@ struct ObjectWalker : public AbstractExpressionVisitor
 		return QString();
 	}
 	
-	virtual QVariant visit(const Apply* c)
+	virtual QString accept ( const Apply* c )
 	{
 		qDebug() << prefix().constData() << "| apply op:" << c->firstOperator().toString();
 		ind++;
@@ -330,7 +291,7 @@ struct ObjectWalker : public AbstractExpressionVisitor
 		return QString();
 	}
 	
-	virtual QVariant visit(const Vector* v)
+	virtual QString accept(const Vector* v)
 	{
 		qDebug() << prefix().constData() << "| vector: " << v->size();
 		ind++;
@@ -340,7 +301,7 @@ struct ObjectWalker : public AbstractExpressionVisitor
 		return QString();
 	}
 	
-	virtual QVariant visit(const List* v)
+	virtual QString accept(const List* v)
 	{
 		qDebug() << prefix().constData() << "| list: " << v->size();
 		ind++;
@@ -350,24 +311,6 @@ struct ObjectWalker : public AbstractExpressionVisitor
 		return QString();
 	}
 	
-	virtual QVariant visit(const Matrix* m) {
-		qDebug() << prefix().constData() << "| matrix: ";
-		ind++;
-		for(Matrix::const_iterator it=m->constBegin(); it!=m->constEnd(); ++it)
-			visitNow(*it);
-		ind--;
-		return QString();
-	}
-	
-	virtual QVariant visit(const MatrixRow* m) {
-		qDebug() << prefix().constData() << "| matrix: ";
-		ind++;
-		for(MatrixRow::const_iterator it=m->constBegin(); it!=m->constEnd(); ++it)
-			visitNow(*it);
-		ind--;
-		return QString();
-	}
-	
 	QByteArray prefix()
 	{
 		QByteArray ret(m_prefix);
@@ -376,9 +319,9 @@ struct ObjectWalker : public AbstractExpressionVisitor
 		return ret;
 	}
 	
-	void visitNow(const Object* o) { if(o) o->accept(this); else qDebug() << prefix().constData() << "Null" ;}
+	void visitNow(const Object* o) { if(o) o->visit(this); else qDebug() << prefix().constData() << "Null" ;}
 	
-	QVariant result() const { return QString(); }
+	QString result() const { return QString(); }
 	
 	int ind;
 	QByteArray m_prefix;
@@ -399,7 +342,7 @@ void objectWalker(const Object* root, const QByteArray& prefix)
 
 bool equalTree(const Object * o1, const Object * o2)
 {
-	Q_ASSERT((o1 && o2) || (!o1 && !o2));
+	Q_ASSERT(o1 && o2);
 	if(o1==o2)
 		return true;
 	else if(o1->type()!=o2->type())
@@ -431,12 +374,6 @@ bool equalTree(const Object * o1, const Object * o2)
 		case Object::custom:
 			eq = *static_cast<const CustomObject*>(o1)==*static_cast<const CustomObject*>(o2);
 			break;
-		case Object::matrix:
-			eq = *static_cast<const Matrix*>(o1)==*static_cast<const Matrix*>(o2);
-			break;
-		case Object::matrixrow:
-			eq = *static_cast<const MatrixRow*>(o1)==*static_cast<const MatrixRow*>(o2);
-			break;
 		case Object::none:
 			eq=false;
 			Q_ASSERT(false && "Should not get here");
@@ -483,9 +420,7 @@ QVariant expressionToVariant(const Analitza::Expression& res)
 
 Analitza::Expression variantToExpression(const QVariant& v)
 {
-	if(v.type() == QVariant::String)
-		return Analitza::Expression(v.toString());
-	else if(v.canConvert(QVariant::Double))
+	if(v.canConvert(QVariant::Double))
 		return Analitza::Expression(Analitza::Cn(v.toReal()));
 	else if(v.canConvert(QVariant::List)) {
 		QVariantList list = v.toList();
@@ -496,9 +431,8 @@ Analitza::Expression variantToExpression(const QVariant& v)
 		}
 		
 		return Analitza::Expression::constructList(expressionList);
-	} else if(v.canConvert<QObject*>()) {
-		return Analitza::Expression::constructCustomObject(v, 0);
-	}
+	} else if(v.canConvert(QVariant::String))
+		return Analitza::Expression(v.toString());
 	
 	Q_ASSERT(false && "couldn't figure out the type");
 	return Analitza::Expression();
@@ -517,7 +451,7 @@ T* replaceDepthTemplate(int depth, T* tree, Object* towhat)
 {
 	Tit it=tree->begin(), itEnd=tree->end();
 	for(; it!=itEnd; ++it)
-		*it = replaceDepth(depth, *it, towhat);
+		*it=replaceDepth(depth, *it, towhat);
 	return tree;
 }
 
@@ -537,10 +471,6 @@ Object* replaceDepth(int depth, Object* tree, Object* towhat)
 			return replaceDepthTemplate<List, List::iterator>(depth, static_cast<List*>(tree), towhat);
 		case Object::vector:
 			return replaceDepthTemplate<Vector, Vector::iterator>(depth, static_cast<Vector*>(tree), towhat);
-		case Object::matrix:
-			return replaceDepthTemplate<Matrix, Matrix::iterator>(depth, static_cast<Matrix*>(tree), towhat);
-		case Object::matrixrow:
-			return replaceDepthTemplate<MatrixRow, MatrixRow::iterator>(depth, static_cast<MatrixRow*>(tree), towhat);
 		case Object::container:
 			return replaceDepthTemplate<Container, Container::iterator>(depth, static_cast<Container*>(tree), towhat);
 		case Object::variable: {
@@ -591,10 +521,6 @@ int countDepth(int depth, const Object* tree)
 			return countDepthTemplate<List, List::const_iterator>(depth, static_cast<const List*>(tree));
 		case Object::vector:
 			return countDepthTemplate<Vector, Vector::const_iterator>(depth, static_cast<const Vector*>(tree));
-		case Object::matrix:
-			return countDepthTemplate<Matrix, Matrix::const_iterator>(depth, static_cast<const Matrix*>(tree));
-		case Object::matrixrow:
-			return countDepthTemplate<MatrixRow, MatrixRow::const_iterator>(depth, static_cast<const MatrixRow*>(tree));
 		case Object::container:
 			return countDepthTemplate<Container, Container::const_iterator>(depth, static_cast<const Container*>(tree));
 		case Object::variable: {
